/*
 *  Gtk-Parted - a front-end for libparted
 *  Copyright (C) 2000, 2001  Damien Genet
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * generic GUI functions
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "i18n.h"
#include "ui.h"

/* pixmaps */
#include "../pixmaps/penguin.xpm"

/*
 * create pixmap from file
 */
void
ui_pixmap_init                         (GtkWidget        *container,
                                        GdkPixmap       **pixmap,
                                        GdkBitmap       **mask,
                                        char             *xpm[])
{
	g_return_if_fail (container);
	g_return_if_fail (GTK_IS_CONTAINER (container));
	g_return_if_fail (pixmap);
	g_return_if_fail (mask);
	g_return_if_fail (xpm);
	
	if (!*pixmap || !*mask) {
		GdkColor	  transparent;
	
		/* dialog MUST be realized before calling gdk_pixmap_create_from_xpm_d */
		gtk_widget_realize (GTK_WIDGET (container));
		*pixmap  = gdk_pixmap_create_from_xpm_d (GTK_WIDGET (container)->window,
		                                    mask,
		                                    &transparent,
		                                    xpm);
	}
}


void
ui_dialog_run                          (GtkDialog        *dialog)
{
	g_return_if_fail (dialog);
	g_return_if_fail (GTK_IS_DIALOG (dialog));
	
	gtk_window_set_modal (GTK_WINDOW (dialog),
	                      TRUE);
	gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
	                                 GTK_SIGNAL_FUNC (gtk_main_quit),
	                                 NULL);
	gtk_main ();
}


GtkWidget*
ui_dialogPopup_create                  (gchar            *message)
{
	GtkWidget	*popup;
	GtkWidget	*label;

	g_return_val_if_fail (message, NULL);

	popup = gtk_window_new (GTK_WINDOW_POPUP);
	gtk_window_set_title (GTK_WINDOW (popup), _("Popup"));
	gtk_window_set_position (GTK_WINDOW (popup), GTK_WIN_POS_MOUSE);
	gtk_window_set_modal (GTK_WINDOW (popup),
	                      TRUE);
	gtk_container_set_border_width (GTK_CONTAINER (popup),
	                                10);
	
	label = gtk_label_new (message);
	gtk_container_add (GTK_CONTAINER (popup), label);
	
	gtk_widget_show_all (GTK_WIDGET (popup));
	
	return popup;
}



static GtkWidget*
_dialog_create_real                    (gchar            *message,
                                        gboolean          wrap)
{
	GtkWidget	*dialog;
	GtkWidget	*hbox;
	GtkWidget	*pixmapLogo;
	GtkWidget	*label;
	static GdkPixmap	*logo = NULL;
	static GdkBitmap	*logo_mask = NULL;

	g_return_val_if_fail (message, NULL);

	dialog = gtk_dialog_new ();
	gtk_window_set_title (GTK_WINDOW (dialog), _("Message"));
	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);

	hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (hbox), 15);

	ui_pixmap_init (GTK_WIDGET (hbox), &logo, &logo_mask, penguin_xpm);
	pixmapLogo =  gtk_pixmap_new (logo, logo_mask);
	gtk_box_pack_start (GTK_BOX (hbox), pixmapLogo, FALSE, TRUE, 10);

	label = gtk_label_new (message);
	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
	gtk_label_set_justify (GTK_LABEL (label),
	                       GTK_JUSTIFY_LEFT);
	gtk_label_set_line_wrap (GTK_LABEL (label),
	                         wrap);
	
	gtk_signal_connect (GTK_OBJECT (dialog), "delete_event",
	                    GTK_SIGNAL_FUNC (gtk_widget_destroy),
	                    NULL);

	return dialog;
}

/*
 * simple dialog message with a OK button
 */
GtkWidget*
ui_dialogMessage_create                (gchar            *message,
                                        gboolean          wrap)
{
	GtkWidget	*dialog;
	GtkWidget	*buttonOk;
	
	g_return_val_if_fail (message, NULL);
	
	if ( !(dialog = _dialog_create_real (message, wrap)) )
		return NULL;

	buttonOk = gtk_button_new_with_label (_("OK"));
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), buttonOk, TRUE, TRUE, 0);
	gtk_button_set_relief (GTK_BUTTON (buttonOk), GTK_RELIEF_NONE);
	GTK_WIDGET_SET_FLAGS (buttonOk, GTK_CAN_DEFAULT);

	gtk_signal_connect_object (GTK_OBJECT (buttonOk), "clicked",
	                    GTK_SIGNAL_FUNC (gtk_widget_destroy),
	                    GTK_OBJECT (dialog));

	gtk_widget_grab_default (GTK_WIDGET (buttonOk));
	gtk_widget_show_all(GTK_WIDGET (dialog));
	return dialog;
}

/*
 * simple confirm dialog with OK CANCEL buttons
 */
GtkWidget*
ui_dialogConfirm_create                (gchar            *message,
                                        GtkSignalFunc     ok_callback)
{
	GtkWidget	*dialog;
	GtkWidget	*hbuttonbox;
	GtkWidget	*buttonOk;
	GtkWidget	*buttonCancel;

	g_return_val_if_fail (message, NULL);
	g_return_val_if_fail (ok_callback, NULL);

	dialog = gtk_dialog_new ();

	if ( !(dialog = _dialog_create_real (message, TRUE)) )
		return NULL;

	hbuttonbox = gtk_hbutton_box_new ();
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), hbuttonbox, TRUE, TRUE, 0);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox), GTK_BUTTONBOX_END);

	buttonOk = gtk_button_new_with_label (_("OK"));
	gtk_container_add (GTK_CONTAINER (hbuttonbox), buttonOk);
	gtk_button_set_relief (GTK_BUTTON (buttonOk), GTK_RELIEF_NONE);
	GTK_WIDGET_SET_FLAGS (buttonOk, GTK_CAN_DEFAULT);

	buttonCancel = gtk_button_new_with_label (_("Cancel"));
	gtk_container_add (GTK_CONTAINER (hbuttonbox), buttonCancel);
	gtk_button_set_relief (GTK_BUTTON (buttonCancel), GTK_RELIEF_NONE);
	GTK_WIDGET_SET_FLAGS (buttonCancel, GTK_CAN_DEFAULT);

	gtk_signal_connect (GTK_OBJECT (buttonOk), "clicked",
	                    GTK_SIGNAL_FUNC (ok_callback),
	                    NULL);
	gtk_signal_connect_object_after (GTK_OBJECT (buttonOk), "clicked",
	                    GTK_SIGNAL_FUNC (gtk_widget_destroy),
	                    GTK_OBJECT (dialog));
	gtk_signal_connect_object (GTK_OBJECT (buttonCancel), "clicked",
	                    GTK_SIGNAL_FUNC (gtk_widget_destroy),
	                    GTK_OBJECT (dialog));

	gtk_widget_grab_default (GTK_WIDGET (buttonCancel));
	gtk_widget_show_all(GTK_WIDGET (dialog));
	
	return dialog;
}


/*
 * create and run dialog_opt
 *   when run, stop execution flow
 *   and return dialog_option num corresponding to button clicked
 */


static void
_dialog_opt_get_clicked_button         (GtkButton        *button,
                                        dialog_option    *button_opt_clicked)
{
	GtkDialog	*dialog;
	dialog_option	 button_opt;

	g_return_if_fail (button);
	g_return_if_fail (GTK_IS_BUTTON (button));
	g_return_if_fail (button_opt_clicked);

	
	dialog = GTK_DIALOG (gtk_widget_get_toplevel (GTK_WIDGET (button)));

	button_opt = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (button),
	                                                    "button_opt"));

	if (!*button_opt_clicked)	/* keep only first if multiple events */
		*button_opt_clicked = button_opt;
}


static gchar*
option_get_string                      (dialog_option     button_opt)
{
	static gchar* option_strings [] = {
		N_("Fix"),
		N_("Yes"),
		N_("No"),
		N_("Ok"),
		N_("Retry"),
		N_("Ignore"),
		N_("Cancel")
	};

	g_return_val_if_fail (button_opt, NULL);
	
	switch (button_opt) {
		case DIALOG_FIX:
			return _(option_strings[0]);
			break;
		case DIALOG_YES:
			return _(option_strings[1]);
			break;
		case DIALOG_NO:
			return _(option_strings[2]);
			break;
		case DIALOG_OK:
			return _(option_strings[3]);
			break;
		case DIALOG_RETRY:
			return _(option_strings[4]);
			break;
		case DIALOG_IGNORE:
			return _(option_strings[5]);
			break;
		case DIALOG_CANCEL:
			return _(option_strings[6]);
			break;
		default:	/* should never arrive here */
			g_assert_not_reached ();
			break;
	}

	return NULL;
}


static GtkWidget*
_dialog_opt_add_button                 (GtkHButtonBox    *hbuttonbox,
                                        dialog_option     button_opt,
                                        dialog_option    *button_opt_clicked)
{
	GtkDialog	*dialog;
	GtkWidget	*button;
	
	g_return_val_if_fail (hbuttonbox, NULL);
	g_return_val_if_fail (GTK_IS_HBUTTON_BOX (hbuttonbox), NULL);
	g_return_val_if_fail (button_opt, NULL);
	g_return_val_if_fail (button_opt_clicked, NULL);
	
	button = gtk_button_new_with_label (option_get_string (button_opt));
	gtk_container_add (GTK_CONTAINER (hbuttonbox), button);
	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);

	gtk_object_set_data (GTK_OBJECT (button),
	                     "button_opt",
	                     GINT_TO_POINTER (button_opt));

	dialog = GTK_DIALOG (gtk_widget_get_toplevel (GTK_WIDGET (hbuttonbox)));

	gtk_signal_connect (GTK_OBJECT (button), "clicked",
	                    GTK_SIGNAL_FUNC (_dialog_opt_get_clicked_button),
	                    button_opt_clicked);
	gtk_signal_connect_object_after (GTK_OBJECT (button), "clicked",
	                    GTK_SIGNAL_FUNC (gtk_widget_destroy),
	                    GTK_OBJECT (dialog));
	return button;
}

static dialog_option
option_get_next                        (dialog_option     options,
                                        dialog_option     current)
{
	dialog_option i;
	
	g_return_val_if_fail (options, 0);
	
	if (current == 0)
		i = DIALOG_OPTION_FIRST;
	else
		i = current * 2;
	
	for (; i <= options; i *= 2) {
		if (options & i)
			return i;
	}
	
	return 0;
}

dialog_option
ui_dialog_opt_create_and_run           (gchar            *message,
                                        dialog_option     options)
{
	GtkWidget	*dialog;
	GtkWidget	*hbuttonbox;
	GtkWidget	*last = NULL;
	dialog_option	 button_opt_clicked = 0;
	dialog_option	 current = 0;

	g_return_val_if_fail (message, 0);
	g_return_val_if_fail (options, 0);

	dialog = gtk_dialog_new ();

	if ( !(dialog = _dialog_create_real (message, TRUE)) )
		return 0;

	hbuttonbox = gtk_hbutton_box_new ();
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), hbuttonbox, TRUE, TRUE, 0);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox), GTK_BUTTONBOX_END);

	while ( (current = option_get_next (options, current)) ) 
		last = _dialog_opt_add_button (GTK_HBUTTON_BOX (hbuttonbox),
		                              current,
		                              &button_opt_clicked);
	
	gtk_widget_grab_default (GTK_WIDGET (last));
	gtk_widget_show_all(GTK_WIDGET (dialog));

	gtk_signal_connect (GTK_OBJECT (dialog), "delete-event",
	                    GTK_SIGNAL_FUNC (gtk_widget_destroy),
	                    NULL);

	ui_dialog_run (GTK_DIALOG (dialog));
	
	return button_opt_clicked;
}
